declare module "action-status.enum" {
    export enum ActionStatus {
        BUSY = "busy",
        SUCCESS = "success",
        FAILURE = "failure"
    }
    export namespace ActionStatus {
        function has(value: string): boolean;
    }
}
declare module "status-selector.interface" {
    import { ActionStatus } from "action-status.enum";
    export interface StatusSelector {
        (state: any): ActionStatus | boolean | null | undefined;
    }
}
declare module "promise-resolver" {
    import type { MiddlewareAPI, Dispatch, AnyAction } from 'redux';
    import type { ThunklessAction } from "thunkless-action.type";
    export type ChainedValue = AnyAction | AnyAction[] | void;
    export type ThunklessChain<T = any> = ChainedValue | Promise<ChainedValue> | ((payload: T) => ChainedValue) | ((payload: T) => Promise<ChainedValue>);
    export type ErrorPayload = {
        error: Error;
        origin: ThunklessAction<any, any>;
    };
    export const createPromiseResolver: (store: MiddlewareAPI) => (next: Dispatch<AnyAction>, promise: Promise<unknown>, action: ThunklessAction<any, any>, successType: string, failureType: string, chain: ThunklessChain, dispatchOnError: ThunklessAction<any, any>['dispatchOnError'], extra: Record<string, any>) => Promise<{
        payload: any;
        type: string;
    } | {
        type: string;
        error: boolean;
        payload: ErrorPayload;
    }>;
}
declare module "thunkless-action.type" {
    import type { AnyAction } from 'redux';
    import type { StatusSelector } from "status-selector.interface";
    import type { ThunklessChain, ErrorPayload } from "promise-resolver";
    export interface ThunklessActionProps<P = never, M = never, S = any, T extends readonly [string, string, string] | string = readonly [string, string, string] | string, R extends AnyAction = AnyAction> {
        type: T;
        error?: boolean;
        payload?: any;
        meta?: M;
        promise?: P extends never ? never : Promise<P> | (() => Promise<P>);
        statusSelector?: P extends never ? never : StatusSelector | StatusSelector[];
        chain?: P extends never ? never : ThunklessChain<P>;
        dispatchOnError?: P extends never ? never : string | ((payload: ErrorPayload) => AnyAction | null | void);
        transform?: (action: this, store: S) => R;
    }
    export type ThunklessAction<P = never, M = never, S = any, T extends readonly [string, string, string] | string = readonly [string, string, string] | string, R extends AnyAction = AnyAction, O extends Record<string, any> = {}> = ThunklessActionProps<P, M, S, T, R> & {
        [K in keyof O]: O[K];
    };
}
declare module "create-thunkless-action" {
    import type { AnyAction } from 'redux';
    import type { ThunklessAction } from "thunkless-action.type";
    export const createThunklessAction: <P = never, M = never, S = any, T extends string | readonly [string, string, string] = [string, string, string], R extends AnyAction = AnyAction, O extends Record<string, any> = {}>(action: ThunklessAction<P, M, S, T, R, O>) => ThunklessAction<P, M, S, T, R, O>;
}
declare module "middleware" {
    import type { MiddlewareAPI, Dispatch, AnyAction } from 'redux';
    import type { ThunklessAction } from "thunkless-action.type";
    export const middleware: (store: MiddlewareAPI) => (next: Dispatch<AnyAction>) => (action: ThunklessAction<any, any>) => AnyAction | Promise<{
        payload: any;
        type: string;
    } | {
        type: string;
        error: boolean;
        payload: import("promise-resolver").ErrorPayload;
    }>;
}
declare module "reducible-thunkless-action.type" {
    import type { ThunklessAction } from "thunkless-action.type";
    import type { ErrorPayload } from "promise-resolver";
    type TransformedAction<A extends ThunklessAction<any, any>, T> = T extends ThunklessAction['transform'] ? ReturnType<T> : A;
    type NeverProps = {
        promise?: never;
        statusSelector?: never;
        chain?: never;
        dispatchOnError?: never;
        transform?: never;
    };
    type MetaProp<T extends ThunklessAction<any, any>> = T['meta'] extends never ? {} : {
        meta: T['meta'];
    };
    type OtherActionProps<T extends ThunklessAction<any, any>> = Omit<T, 'type' | 'payload' | 'meta' | 'promise' | 'statusSelector' | 'chain' | 'dispatchOnError' | 'transform'>;
    type SuccessPayload<T extends ThunklessAction<any, any>> = T['promise'] extends Promise<infer R> ? R : T['promise'] extends (...args: any[]) => Promise<infer R> ? R : T['promise'] extends Promise<infer R> | ((...args: any[]) => Promise<infer R>) ? R : never;
    export type ReducibleThunklessAction<T extends ThunklessAction<any, any>> = T['type'] extends readonly [string, string, string] ? (TransformedAction<{
        type: T['type'][0];
        payload?: T['payload'];
    } & MetaProp<T> & NeverProps & {
        [K in keyof OtherActionProps<T>]: T[K];
    }, T['transform']> | TransformedAction<{
        type: T['type'][1];
        payload: SuccessPayload<T>;
    } & MetaProp<T> & NeverProps & {
        [K in keyof OtherActionProps<T>]: T[K];
    }, T['transform']> | TransformedAction<{
        type: T['type'][2];
        payload: ErrorPayload;
        error: true;
    } & MetaProp<T> & NeverProps & {
        [K in keyof OtherActionProps<T>]: T[K];
    }, T['transform']>) : T;
}
declare module "thunkless" {
    export { middleware } from "middleware";
    export { createThunklessAction } from "create-thunkless-action";
    export { ActionStatus } from "action-status.enum";
    export { ChainedValue, ThunklessChain, ErrorPayload } from "promise-resolver";
    export { StatusSelector } from "status-selector.interface";
    export { ThunklessAction } from "thunkless-action.type";
    export { ReducibleThunklessAction } from "reducible-thunkless-action.type";
}
//# sourceMappingURL=thunkless.d.ts.map